package jp.co.celestee.ai.interfaces.image;

import java.awt.Color;
import java.awt.Point;
import java.awt.Polygon;
import java.util.ArrayList;
import java.util.List;

import jp.co.celestee.ai.core.signal.AbstractSignalData;
import jp.co.celestee.ai.core.signal.SignalDataIF;
import jp.co.celestee.ai.interfaces.SignalParts;

/**
 * 摜̂̂̃f[^łB
 * @author celestee
 *
 */
public class ImageSignalData extends AbstractSignalData {

	private static final long serialVersionUID = 1L;

	/** FƂ\ */
	private static final Color COLOR_NULL = Color.WHITE;
	
	/** `Ƃ\ */
	private static final Polygon POLYGON_NULL = PolygonNull.getInstance();

	/** ΏەȂƂ\ */
	private static final List<Point> POINT_NULL = new ArrayList<Point>(1);
	
	/** F */
	@SignalParts
	private Color color = null;
	
	/** ` */
	@SignalParts
	private Polygon polygon = null;
	
	/** `̈ʒuXg */
	@SignalParts
	private List<Point> pointList = null;
	
	@SignalParts
	private int pointCount = 0;
	
	public ImageSignalData() {
	}
	
	public ImageSignalData(Color color, Polygon polygon, List<Point> pointList) {
		setColor(color);
		setPolygon(polygon);
		//ȉ̏2s1ZbgōsƁBۉ̏ꍇ́AsetPointListƃJEg͈vȂ̂ŒӁB
		setPointList(pointList);
		setPointCount(pointList == null ? 0 : pointList.size());
	}
	
	@Override
	public Class<?> getOutputIdClass() {
		return ImageOutputId.class;
	}
	
	public Color getColor() {
		return color;
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public Polygon getPolygon() {
		return polygon;
	}

	public void setPolygon(Polygon polygon) {
		this.polygon = polygon;
	}

	public List<Point> getPointList() {
		return pointList;
	}

	/**
	 * |CgXgۂƓւ܂BAA|CgJEg͍XV܂B
	 * pŃ|CgJEg̍XVsĂiۉ̏ꍇƁAۉ̏ꍇł̓JEgݒeقȂׁj
	 * @param pointList
	 */
	public void setPointList(List<Point> pointList) {
		this.pointList = pointList;
	}

	/**
	 * |CgXgɎw̃|CgXgǉ܂B܂A|CgJEgZ܂B
	 * @param pointList
	 */
	public void addPointList(List<Point> pointList) {
		for (Point point : pointList) {
			addPoint(point);
		}
	}

	/**
	 * |CgXgɎw̃|Cgǉ܂B܂A|CgJEgZ܂B
	 * @param point
	 */
	public void addPoint(Point point) {
		if(this.pointList.contains(point)) {
			return;
		}
		else {
			this.pointList.add(point);
			pointCount++;
		}
	}

	public int getPointCount() {
		return pointCount;
	}

	void setPointCount(int count) {
		pointCount = count;
	}

	public static Color colorFreeValueReturn() {
		return COLOR_NULL;
	}

	public static Polygon polygonFreeValueReturn() {
		return POLYGON_NULL;
	}

	public static int pointCountFreeValueReturn() {
		return -1;
	}
	
	public static List<Point> pointListFreeValueReturn() {
		return POINT_NULL;
	}

	@Override
	public String toString() {
		String sColor = null;
		if(color == COLOR_NULL) {
			sColor = "NONE_COLOR";
		}
		else {
			sColor = color.toString();
		}
		
		return "(COLOR=" + sColor + " ,POLYGON=" + polygon + " ,POINT=" + getPointCount(pointList) + " ,COUNT=" + pointCount + " ,In=" + getInner() + ")";
	}
	
	private String getPointCount(List<Point> pList) {
		if(pList == null || pList.size() == 0) {
			return "";
		}
		
		StringBuilder sb = new StringBuilder();
		for(Point point : pList) {
			sb.append("<" + point.x + "," + point.y + ">");
		}
		
		return sb.toString();
	}

	@Override
	public List<SignalDataIF> getPartsSignalDataList() {
		List<SignalDataIF> signalList = new ArrayList<SignalDataIF>(3);
		
		ImageSignalData dataC = new ImageSignalData(color, POLYGON_NULL, null);
		signalList.add(dataC);
		
		ImageSignalData dataP = new ImageSignalData(COLOR_NULL, polygon, null);
		signalList.add(dataP);
		
		ImageSignalData dataL = new ImageSignalData(COLOR_NULL, POLYGON_NULL, pointList);
		signalList.add(dataL);
		
		return signalList;
	}
	
	@Override
	public boolean isChain(SignalDataIF targetSignalData) {
		return false;
	}
	
	@Override
	public boolean isLooklike(SignalDataIF targetSingalData) {
		if(super.isLooklike(targetSingalData)) {
			return true;
		}
		
		return sameData(targetSingalData);
	}

	@Override
	public AbstractSignalData copyInstance() {
		AbstractSignalData data = super.copyInstance();

		if(pointList != null) {
			List<Point> copyPointList = new ArrayList<Point>(pointList.size());
			for(Point point : pointList) {
				copyPointList.add(new Point(point.x, point.y));
			}
			
			ImageSignalData imageSignalData = (ImageSignalData)data;
			//ȉ̏2s1ZbgōsƁBۉ̏ꍇ́AsetPointListƃJEg͈vȂ̂ŒӁB
			imageSignalData.setPointList(copyPointList);
			imageSignalData.setPointCount(pointCount);
		}
		
		return data;
	}

	@Override
	public boolean equals(Object target) {
		boolean b = super.equals(target);
		if(b == false) return b;
		
		return target instanceof SignalDataIF ? sameData((SignalDataIF)target) : false;
	}

	@Override
	public int hashCode() {
		return 0;
	}

	@Override
	public String getSupposingItSaysInMansWords() {
		return "looking " + getColor() + " , " + getPolygon() + " , " + getPointList() + "," + pointCount + "count";
	}

}
