package jp.co.celestee.ai.interfaces.image;

import java.awt.Color;
import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

import jp.co.celestee.ai.interfaces.AbstractSubstance;
import jp.co.celestee.ai.interfaces.EntranceReaction;
import jp.co.celestee.ai.interfaces.input.DefaultDataCooperationSignal;
import jp.co.celestee.ai.interfaces.status.JAbstractedImage;
import jp.co.celestee.ai.interfaces.status.JFrontRetina;
import jp.co.celestee.ai.interfaces.status.JRetina;
import jp.co.celestee.external.rule.RuleDataIF;
import jp.co.celestee.external.rule.light.LightRuleAccessorIF;
import jp.co.celestee.external.rule.light.LightRuleData;

/**
 * 摜IF̖{̏łB
 * @author celestee
 *
 */
public class ImageSubstance extends AbstractSubstance implements LightRuleAccessorIF {

	private static final int NORMAL_MODE = 0;
	private static final int ADD_MODE = 1;
	private static final int MODE_KEEP_TIME = 10000;
	
	private static ImageSubstance subInstance = new ImageSubstance();
	private static ImageSubstance simulateSubInstance = new ImageSubstance();
	
	private ImageSignalData _retinaImage = null;
	private int _mode = NORMAL_MODE;
	private long _change_add_mode_time = 0;
	
	private ImageSubstance() {}

	/**
	 * ImageSubstance擾
	 * @return
	 */
	public static ImageSubstance getInstance() {
		return subInstance;
	}

	/**
	 * V~[VpImageSubstance擾
	 * @return
	 */
	public static ImageSubstance getSimulateInstance() {
		return simulateSubInstance;
	}

	/**
	 * ԖC[W擾܂
	 * @return
	 */
	public ImageSignalData getRetinaImage() {
		return _retinaImage;
	}

	/**
	 * ԖC[Wݒ肵܂
	 * @param imageSignalData
	 * @return
	 */
	public ImageSignalData setRetinaImage(ImageSignalData imageSignalData) {
		return _retinaImage = imageSignalData;
	}

	/**
	 * Ԗˉe[h_ړiǉj[hɕύX܂B
	 */
	public void changeModeAdd() {
		_mode = ADD_MODE;
		_change_add_mode_time = System.currentTimeMillis();
	}

	/**
	 * Ԗˉe[hʏ탂[hɕύX܂B
	 */
	public void changeModeNormal() {
		_mode = NORMAL_MODE;
	}
	
	/**
	 * [IFւ̓
	 */
	@Override
	public void inputLight(LightRuleData ruleData) {
		inputDataFromRule(ruleData);
	}

	/**
	 * [IFւ̓
	 */
	@Override
	public void inputFromRule(RuleDataIF ruleData, Signals signals) {

		//IFŃf[^ϊiዅŌԖ̓Ɠj
		LightRuleData imageRuleData = (LightRuleData)ruleData;
		DefaultDataCooperationSignal dcSignal = ImageDataConverter.change(imageRuleData);
		inputConvertCommon(dcSignal);
		
		signals.setDataCooperationSignal(dcSignal);
		//System.out.println("+++ " + dcSignal.getSignalData());
	}

	@Override
	public void inputFromAI(Object aiData, Signals signals) {
		//AI-IFԂŎvoĂ
		inputConvertCommon((DefaultDataCooperationSignal)aiData);
		signals.setDataCooperationSignal((DefaultDataCooperationSignal)aiData);
	}

	private void inputConvertCommon(DefaultDataCooperationSignal dcSignal) {
		//摜ǉ[h̏IiƓ悤ɁARɖ߂j
		if(System.currentTimeMillis() - _change_add_mode_time > MODE_KEEP_TIME) {
			changeModeNormal();
		}
	}
	
	@Override
	public void process(Signals signals) {
		//ACRjbN[ʂ
		chageMemory((ImageSignalData)signals.getDataCooperationSignal().getSignalData());
		
		//Ԗ摜[Uɕ\
		JRetina retina = JFrontRetina.getInstance();
		retina.setColor(_retinaImage.getColor());
		retina.setPolygon(_retinaImage.getPolygon());
		retina.setCountList(_retinaImage.getPointList());
		retina.setAbstCount(_retinaImage.getPointCount());
		retina.setStartY(_mode);
		retina.repaint();
		
		//ʂ̐ݒ
		signals.getDataCooperationSignal().setSignalData(_retinaImage.copyInstance());

		//[h͏Ƃɖ߂
		changeModeNormal();
	}
	
	/**
	 * Ԗ̃ACRjbN[ύX܂
	 * @param imageSignalData ύXf[^
	 */
	public void chageMemory(ImageSignalData imageSignalData) {

		if(imageSignalData == null) return;

		if(_mode == NORMAL_MODE || _retinaImage == null) {
			//ʏ탂[h(C[Wf[^ւ)
			_retinaImage = (ImageSignalData)imageSignalData.copyInstance();
		}
		else if(_mode == ADD_MODE) {
			//_ړɂ摜ǉ[h

			//ۃC[W̏ꍇ
			if(_retinaImage.getPointList().equals(ImageSignalData.pointListFreeValueReturn()) == false &&
					imageSignalData.getPointList().equals(ImageSignalData.pointListFreeValueReturn()) == false) {

				List<Point> pointList = new ArrayList<Point>();
				synchronized(imageSignalData.getPointList()) {
					for(Point point : imageSignalData.getPointList()) {
						pointList.add(new Point(point.x, point.y + 2));
					}
				}				
				_retinaImage.addPointList(pointList);
				_retinaImage.setAbstract(false);
			}
			//ۃC[W̏ꍇ
			else {
				int abstractCount = _retinaImage.getPointCount() + imageSignalData.getPointCount();
				//ȉ̏2s1ZbgōsƁBۉ̏ꍇ́AsetPointListƃJEg͈vȂ̂ŒӁB
				_retinaImage.setPointList(ImageSignalData.pointListFreeValueReturn());
				_retinaImage.setPointCount(abstractCount);
				_retinaImage.setAbstract(true);
			}
			
		}
	}

	@Override
	public void outputToAI(
			Signals signals
			) {

		ImageSignalData imageSignalData = (ImageSignalData)signals.getDataCooperationSignal().getSignalData();
		//ۉ
		doAbstract(imageSignalData);

		//ۉ摜[Uɕ\
		JAbstractedImage abstImage = JAbstractedImage.getInstance();
		abstImage.setColor(imageSignalData.getColor());
		abstImage.setPolygon(imageSignalData.getPolygon());
		abstImage.setCountList(imageSignalData.getPointList());
		abstImage.setAbstCount(imageSignalData.getPointCount());
		abstImage.setStartY(_mode);
		abstImage.repaint();

		try{
			//AI֓dCM𑗂
			EntranceReaction.startReaction(
					signals.getDataCooperationSignal(),
					signals.getEnvironmentSensibleSignal(),
					signals.getPhysicalSensibleSignalList());
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}


	@Override
	public void outputToRule(Signals signals) {
		//[ւ̏o͂͂Ȃ

	}
	
	/**
	 * AI߂f[^ւ̕ϊƁAۉsB_ł́AȈՓIȎ݂̂ƂBʓIɌĂAȑgD}bv̎_ł́A
	 * ōsBAAICoreŎȑgD}bv邩͌B
	 * @param signalData
	 */
	private void doAbstract(ImageSignalData signalData) {
		//polygon̒ۉ(ȈՎ)
		signalData.setPolygon(PolygonAbstractSample.getInstance());
	}

	/**
	 * IWFɈꎞIɐݒ肷
	 */
	public void changeColor() {
		JAbstractedImage abstImage = JAbstractedImage.getInstance();
		abstImage.setColor(Color.blue);
		abstImage.repaint();
	}
}
